@@ -20,7 +20,6 @@ import copy
 import logging
 import os
 import platform
-import re
 import socket
 import warnings
 
@@ -33,8 +32,11 @@ from ibm_botocore.configprovider import ConfigValueStore
 from ibm_botocore.configprovider import ConfigChainFactory
 from ibm_botocore.configprovider import create_botocore_default_config_mapping
 from ibm_botocore.configprovider import BOTOCORE_DEFAUT_SESSION_VARIABLES
+from ibm_botocore.configprovider import DefaultConfigResolver
+# IBM Unsupported
+# from ibm_botocore.configprovider import SmartDefaultsConfigStoreFactory
 from ibm_botocore.exceptions import (
-    ConfigNotFound, ProfileNotFound, UnknownServiceError,
+    ConfigNotFound, ProfileNotFound, UnknownServiceError, InvalidDefaultsMode,
     PartialCredentialsError,
 )
 from ibm_botocore.errorfactory import ClientExceptionsFactory
@@ -49,8 +51,9 @@ from ibm_botocore import monitoring
 from ibm_botocore import paginate
 from ibm_botocore import waiter
 from ibm_botocore import retryhandler, translate
-from ibm_botocore import utils
 from ibm_botocore.utils import EVENT_ALIASES, validate_region_name
+# IBM Unsupported
+# from ibm_botocore.utils import IMDSRegionProvider
 from ibm_botocore.compat import MutableMapping, HAS_CRT
 
 
@@ -139,6 +142,9 @@ class Session(object):
         self._register_exceptions_factory()
         self._register_config_store()
         self._register_monitor()
+        self._register_default_config_resolver()
+        # IBM Unsupported
+        # self._register_smart_defaults_factory()
 
     def _register_event_emitter(self):
         self._components.register_component('event_emitter', self._events)
@@ -155,7 +161,7 @@ class Session(object):
     def _register_data_loader(self):
         self._components.lazy_register_component(
             'data_loader',
-            lambda:  create_loader(self.get_config_variable('data_path')))
+            lambda: create_loader(self.get_config_variable('data_path')))
 
     def _register_endpoint_resolver(self):
         def create_default_resolver():
@@ -165,6 +171,24 @@ class Session(object):
         self._internal_components.lazy_register_component(
             'endpoint_resolver', create_default_resolver)
 
+    def _register_default_config_resolver(self):
+        def create_default_config_resolver():
+            loader = self.get_component('data_loader')
+            defaults = loader.load_data('sdk-default-configuration')
+            return DefaultConfigResolver(defaults)
+        self._internal_components.lazy_register_component(
+            'default_config_resolver', create_default_config_resolver)
+
+    # def _register_smart_defaults_factory(self):
+    #     def create_smart_defaults_factory():
+    #         default_config_resolver = self._get_internal_component(
+    #             'default_config_resolver')
+    #         imds_region_provider = IMDSRegionProvider(session=self)
+    #         return SmartDefaultsConfigStoreFactory(
+    #             default_config_resolver, imds_region_provider)
+    #     self._internal_components.lazy_register_component(
+    #         'smart_defaults_factory', create_smart_defaults_factory)
+
     def _register_response_parser_factory(self):
         self._components.register_component('response_parser_factory',
                                             ResponseParserFactory())
@@ -214,16 +238,13 @@ class Session(object):
             return handler
         return None
 
-    def _get_crt_version(self):
-        try:
-            import pkg_resources
-            return pkg_resources.get_distribution("awscrt").version
-        except Exception:
-            # We're catching *everything* here to avoid failing
-            # on pkg_resources issues. This is unlikely in our
-            # supported versions but it avoids making a hard
-            # dependency on the package being present.
-            return "Unknown"
+    # IBM Unsupported
+    # def _get_crt_version(self):
+    #     try:
+    #         import awscrt
+    #         return awscrt.__version__
+    #     except AttributeError:
+    #         return "Unknown"
 
     @property
     def available_profiles(self):
@@ -496,8 +517,9 @@ class Session(object):
                                           platform.python_version(),
                                           platform.system(),
                                           platform.release())
-        if HAS_CRT:
-            base += ' awscrt/%s' % self._get_crt_version()
+        # IBM Unsupported
+        # if HAS_CRT:
+        #     base += ' awscrt/%s' % self._get_crt_version()
         if os.environ.get('AWS_EXECUTION_ENV') is not None:
             base += ' exec-env/%s' % os.environ.get('AWS_EXECUTION_ENV')
         if self.user_agent_extra:
@@ -901,6 +923,13 @@ class Session(object):
         endpoint_resolver = self._get_internal_component('endpoint_resolver')
         exceptions_factory = self._get_internal_component('exceptions_factory')
         config_store = self.get_component('config_store')
+        defaults_mode = self._resolve_defaults_mode(config, config_store)
+        if defaults_mode != 'legacy':
+            smart_defaults_factory = self._get_internal_component(
+                'smart_defaults_factory')
+            config_store = copy.deepcopy(config_store)
+            smart_defaults_factory.merge_smart_defaults(
+                config_store, defaults_mode, region_name)
         client_creator = ibm_botocore.client.ClientCreator(
             loader, endpoint_resolver, self.user_agent(), event_emitter,
             retryhandler, translate, response_parser_factory,
@@ -958,6 +987,24 @@ class Session(object):
         self._last_client_region_used = region_name
         return region_name
 
+    def _resolve_defaults_mode(self, client_config, config_store):
+        mode = config_store.get_config_variable('defaults_mode')
+
+        if client_config and client_config.defaults_mode:
+            mode = client_config.defaults_mode
+
+        default_config_resolver = self._get_internal_component(
+            'default_config_resolver')
+        default_modes = default_config_resolver.get_default_modes()
+        lmode = mode.lower()
+        if lmode not in default_modes:
+            raise InvalidDefaultsMode(
+                mode=mode,
+                valid_modes=', '.join(default_modes)
+            )
+
+        return lmode
+
     def _missing_cred_vars(self, access_key, secret_key):
         if access_key is not None and secret_key is None:
             return 'aws_secret_access_key'
@@ -974,6 +1021,19 @@ class Session(object):
         resolver = self._get_internal_component('endpoint_resolver')
         return resolver.get_available_partitions()
 
+    def get_partition_for_region(self, region_name):
+        """Lists the partition name of a particular region.
+
+        :type region_name: string
+        :param region_name: Name of the region to list partition for (e.g.,
+             us-east-1).
+
+        :rtype: string
+        :return: Returns the respective partition name (e.g., aws).
+        """
+        resolver = self._get_internal_component('endpoint_resolver')
+        return resolver.get_partition_for_region(region_name)
+
     def get_available_regions(self, service_name, partition_name='aws',
                               allow_non_regional=False):
         """Lists the region and endpoint names of a particular partition.
