@@ -12,8 +12,8 @@
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
 from tests import (
-    unittest, temporary_file, random_chars, ClientHTTPStubber,
-    ConsistencyWaiter,
+    unittest, temporary_file, random_chars,
+    ClientHTTPStubber, ConsistencyWaiter,
 )
 import os
 import time
@@ -22,14 +22,12 @@ import tempfile
 import shutil
 import threading
 import logging
-import mock
 from tarfile import TarFile
 from contextlib import closing
 
-from nose.plugins.attrib import attr
+import pytest
 import urllib3
 
-from ibm_botocore.endpoint import Endpoint
 from ibm_botocore.exceptions import ConnectionClosedError
 from ibm_botocore.compat import six, zip_longest, OrderedDict
 import ibm_botocore.session
@@ -324,7 +322,7 @@ class TestS3Objects(TestS3BaseWithBucket):
             Bucket=self.bucket_name, Key=key_name)
         self.assert_status_code(response, 204)
 
-    @attr('slow')
+    @pytest.mark.slow
     def test_can_paginate(self):
         for i in range(5):
             key_name = 'key%s' % i
@@ -340,7 +338,7 @@ class TestS3Objects(TestS3BaseWithBucket):
                      for el in responses]
         self.assertEqual(key_names, ['key0', 'key1', 'key2', 'key3', 'key4'])
 
-    @attr('slow')
+    @pytest.mark.slow
     def test_can_paginate_with_page_size(self):
         for i in range(5):
             key_name = 'key%s' % i
@@ -357,7 +355,7 @@ class TestS3Objects(TestS3BaseWithBucket):
                      for el in data]
         self.assertEqual(key_names, ['key0', 'key1', 'key2', 'key3', 'key4'])
 
-    @attr('slow')
+    @pytest.mark.slow
     def test_result_key_iters(self):
         for i in range(5):
             key_name = 'key/%s/%s' % (i, i)
@@ -380,7 +378,7 @@ class TestS3Objects(TestS3BaseWithBucket):
         self.assertIn('Contents', response)
         self.assertIn('CommonPrefixes', response)
 
-    @attr('slow')
+    @pytest.mark.slow
     def test_can_get_and_put_object(self):
         self.create_object('foobarbaz', body='body contents')
         time.sleep(3)
@@ -504,8 +502,9 @@ class TestS3Objects(TestS3BaseWithBucket):
         self.assertEqual(len(parsed['Contents']), 1)
         self.assertEqual(parsed['Contents'][0]['Key'], key_name)
 
-        parsed = self.client.list_objects_v2(Bucket=self.bucket_name,
-                                          EncodingType='url')
+        parsed = self.client.list_objects_v2(
+            Bucket=self.bucket_name, EncodingType='url'
+        )
         self.assertEqual(len(parsed['Contents']), 1)
         self.assertEqual(parsed['Contents'][0]['Key'], 'foo%08')
 
@@ -519,8 +518,9 @@ class TestS3Objects(TestS3BaseWithBucket):
         self.assertEqual(len(parsed['Versions']), 1)
         self.assertEqual(parsed['Versions'][0]['Key'], key_name)
 
-        parsed = self.client.list_object_versions(Bucket=self.bucket_name,
-                                          EncodingType='url')
+        parsed = self.client.list_object_versions(
+            Bucket=self.bucket_name, EncodingType='url'
+        )
         self.assertEqual(len(parsed['Versions']), 1)
         self.assertEqual(parsed['Versions'][0]['Key'], 'foo%03')
 
@@ -930,7 +930,7 @@ class TestS3SigV4Client(BaseS3ClientTest):
                                               Key='foo.txt', Body=body)
             self.assert_status_code(response, 200)
 
-    @attr('slow')
+    @pytest.mark.slow
     def test_paginate_list_objects_unicode(self):
         key_names = [
             u'non-ascii-key-\xe4\xf6\xfc-01.txt',
@@ -953,7 +953,7 @@ class TestS3SigV4Client(BaseS3ClientTest):
 
         self.assertEqual(key_names, key_refs)
 
-    @attr('slow')
+    @pytest.mark.slow
     def test_paginate_list_objects_safe_chars(self):
         key_names = [
             u'-._~safe-chars-key-01.txt',
