@@ -15,6 +15,7 @@ from tests import unittest
 import ibm_botocore.session
 from ibm_botocore.exceptions import ClientError
 
+
 class TestSTS(unittest.TestCase):
     def setUp(self):
         self.session = ibm_botocore.session.get_session()
@@ -38,5 +39,5 @@ class TestSTS(unittest.TestCase):
         self.assertEqual(sts.meta.endpoint_url,
                          'https://sts.us-west-2.amazonaws.com')
         # Signing error will be thrown with the incorrect region name included.
-        with self.assertRaisesRegex(ClientError, 'ap-southeast-1') as e:
+        with self.assertRaisesRegex(ClientError, 'ap-southeast-1'):
             sts.get_session_token()
